"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AnimatedStoreScrolling = void 0;

var _reactNativeReanimated = _interopRequireDefault(require("react-native-reanimated"));

var _reactNativeGestureHandler = require("react-native-gesture-handler");

var _utility = require("./utility");

var _AnimatedStoreSheet = require("./AnimatedStoreSheet");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const {
  call,
  cond,
  greaterThan,
  lessThan,
  neq,
  clockRunning,
  not,
  and,
  set,
  sub,
  or,
  stopClock,
  lessOrEq,
  proc,
  add,
  max,
  min,
  eq,
  multiply,
  block,
  onChange,
  Value
} = _reactNativeReanimated.default;

/**
  * Namespace for animated values associated with the scrolling content.
  */
class AnimatedStoreScrolling {}

exports.AnimatedStoreScrolling = AnimatedStoreScrolling;

_defineProperty(AnimatedStoreScrolling, "_scrollingClock", new _reactNativeReanimated.default.Clock());

_defineProperty(AnimatedStoreScrolling, "_scrollY", new _reactNativeReanimated.default.Value(0));

_defineProperty(AnimatedStoreScrolling, "_velocityScrollY", new _reactNativeReanimated.default.Value(0));

_defineProperty(AnimatedStoreScrolling, "_panScrollState", new _reactNativeReanimated.default.Value(_reactNativeGestureHandler.State.END));

_defineProperty(AnimatedStoreScrolling, "_onGestureEventScrolling", _reactNativeReanimated.default.event([{
  nativeEvent: {
    translationY: AnimatedStoreScrolling._scrollY,
    velocityY: AnimatedStoreScrolling._velocityScrollY,
    state: AnimatedStoreScrolling._panScrollState
  }
}]));

_defineProperty(AnimatedStoreScrolling, "enabledContentGestureInteraction", true);

_defineProperty(AnimatedStoreScrolling, "contentHeight", new _reactNativeReanimated.default.Value(0));

_defineProperty(AnimatedStoreScrolling, "headerHeight", 0);

_defineProperty(AnimatedStoreScrolling, "snapPoints", []);

_defineProperty(AnimatedStoreScrolling, "init", (enabledContentGestureInteraction, snapPoints) => {
  AnimatedStoreScrolling.snapPoints = snapPoints;
  AnimatedStoreScrolling.enabledContentGestureInteraction = enabledContentGestureInteraction;
});

_defineProperty(AnimatedStoreScrolling, "handleLayoutHeader", ({
  nativeEvent: {
    layout: {
      height: heightOfHeader
    }
  }
}) => {
  AnimatedStoreScrolling.headerHeight = heightOfHeader;
});

_defineProperty(AnimatedStoreScrolling, "handleLayoutContent", ({
  nativeEvent: {
    layout: {
      height
    }
  }
}) => {
  const resultHeight = AnimatedStoreScrolling.enabledContentGestureInteraction ? height + AnimatedStoreScrolling.headerHeight - AnimatedStoreScrolling.snapPoints[AnimatedStoreScrolling.snapPoints.length - 1] : 0;
  AnimatedStoreScrolling.contentHeight.setValue(Math.max(resultHeight, 0));
});

_defineProperty(AnimatedStoreScrolling, "limitedScroll", proc(val => max(min(val, 0), multiply(AnimatedStoreScrolling.contentHeight, -1))));

_defineProperty(AnimatedStoreScrolling, "_prevTransY", new _reactNativeReanimated.default.Value(0));

_defineProperty(AnimatedStoreScrolling, "_transY", new _reactNativeReanimated.default.Value(0));

_defineProperty(AnimatedStoreScrolling, "_startedAtTheTop", new _reactNativeReanimated.default.Value(0));

_defineProperty(AnimatedStoreScrolling, "_lastState", new _reactNativeReanimated.default.Value(0));

_defineProperty(AnimatedStoreScrolling, "scrollOffset", new _reactNativeReanimated.default.Value(0));

_defineProperty(AnimatedStoreScrolling, "scrollOffsetWhileSnapped", new _reactNativeReanimated.default.Value(0));

_defineProperty(AnimatedStoreScrolling, "_masterScrollY", block([
/** 
  * Debugging section.
  */

/* onChange(
  this._prevTransY,
  call([this._prevTransY], (snapPoints: readonly number[]) =>
    console.log('Changed prevTransY: ' + snapPoints[0])
  )
),
onChange(
  this.scrollOffsetWhileSnapped,
  call([this.scrollOffsetWhileSnapped], (snapPoints: readonly number[]) =>
    console.log('Changed scrollOffsetWhileSnapped: ' + snapPoints[0])
  )
),
onChange(
  this._transY,
  call([this._transY], (snapPoints: readonly number[]) =>
    console.log('Changed scrollOffset: ' + snapPoints[0])
  )
), */

/* onChange(
  this._lastState,
  call([this._lastState], (snapPoints: readonly number[]) =>
    console.log('Changed state: ' + snapPoints[0])
  )
), */
cond(eq(AnimatedStoreScrolling._lastState, 2), [set(_AnimatedStoreSheet.AnimatedStoreSheet._scrollToDragVal, sub(AnimatedStoreScrolling._scrollY, AnimatedStoreScrolling.scrollOffsetWhileSnapped)),
/** 
  * Setting scrollOffset is not required if the bottom sheet is
  * at the very top.
  *  TODO: describe different cases this solution solves
  */
cond(not(AnimatedStoreScrolling._startedAtTheTop), [set(AnimatedStoreScrolling.scrollOffset, sub(AnimatedStoreScrolling._scrollY, AnimatedStoreScrolling.scrollOffsetWhileSnapped))]), set(_AnimatedStoreSheet.AnimatedStoreSheet._velocityY, AnimatedStoreScrolling._velocityScrollY)]), cond(eq(AnimatedStoreScrolling._lastState, 1), [
/** 
  * During scrolling we need to record content offset and then use it when
  * calculating ASBS._scrollToDragVal value, so that dragging doesn't 
  * include unnecessary scrolling position value.
  */
set(AnimatedStoreScrolling.scrollOffsetWhileSnapped, AnimatedStoreScrolling._scrollY),
/** 
  * Contrary to the previous line of code this one covers the special case
  * when we started dragging the bottom sheet not from the top. Here we need to
  * include this._prevTransY as an offset.
  */
cond(not(AnimatedStoreScrolling._startedAtTheTop), set(AnimatedStoreScrolling.scrollOffsetWhileSnapped, multiply(-1, AnimatedStoreScrolling._prevTransY)))]), cond(or(eq(AnimatedStoreScrolling._panScrollState, _reactNativeGestureHandler.State.ACTIVE), eq(AnimatedStoreScrolling._panScrollState, _reactNativeGestureHandler.State.BEGAN)), [// TODO: stop draggin sheet clock
stopClock(AnimatedStoreScrolling._scrollingClock), set(AnimatedStoreScrolling._transY, AnimatedStoreScrolling.limitedScroll(add(AnimatedStoreScrolling._scrollY, AnimatedStoreScrolling._prevTransY, multiply(-1, AnimatedStoreScrolling.scrollOffset)))), AnimatedStoreScrolling._transY], [set(AnimatedStoreScrolling.scrollOffset, 0), set(AnimatedStoreScrolling.scrollOffsetWhileSnapped, 0), set(AnimatedStoreScrolling._scrollY, 0), cond(eq(AnimatedStoreScrolling._lastState, 1), (0, _utility.runDecay)(AnimatedStoreScrolling._scrollingClock, AnimatedStoreScrolling._transY, AnimatedStoreScrolling._velocityScrollY, AnimatedStoreScrolling.contentHeight)), set(AnimatedStoreScrolling._velocityScrollY, 0), set(AnimatedStoreScrolling._prevTransY, AnimatedStoreScrolling._transY), AnimatedStoreScrolling._transY]), AnimatedStoreScrolling._transY]));
//# sourceMappingURL=AnimatedStoreScrolling.js.map